#! /usr/bin/env python3

#
# Copyright (c) 2022 - 2025 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import os
from periphery import I2C
import set_tps546b24a as tps546b24a

#
# This routine reads 'Product Name' field of an EEPROM.
#
def read_product_name(bus, address):
    PRODUCT_NAME_LEN = 16
    PRODUCT_NAME_OFFSET = 0x16

    i2c = I2C(bus)

    # Create an array to hold 'Product Name'
    product_name = [0 for i in range(PRODUCT_NAME_LEN)] 

    # Create message to set the reference address
    msg_ref_address = I2C.Message([PRODUCT_NAME_OFFSET], read = False)

    # Create message to read data
    msg_read = I2C.Message(product_name, read = True)

    # Assemble messages
    msgs = [msg_ref_address, msg_read]

    # Call transfer function
    i2c.transfer(address, msgs)

    i2c.close()
    return ''.join(map(chr, msgs[1].data))[:8]


#
# Main routine
#
DC_EEPROM_BUS = "/dev/i2c-1"
DC_EEPROM_ADDRESS = 0x52

#
# Determine which Daughter Card is plugged-in.
#
name = read_product_name(DC_EEPROM_BUS, DC_EEPROM_ADDRESS)
#print("Daughter Card: ", name)

REGULATOR_BUS = "/dev/i2c-0"

#
# VCCO_500 is at 0x1A, VCCO_501 is at 0x1B, and VCCO_502 is at 0x1C.
#
if name == "X-PRC-07" or name == "X-PRC-08":
    tps546b24a.set_tps546b24a(REGULATOR_BUS, 0x1A, 1.8)
    tps546b24a.set_tps546b24a(REGULATOR_BUS, 0x1B, 1.8)
    tps546b24a.set_tps546b24a(REGULATOR_BUS, 0x1C, 3.2)
else:
    tps546b24a.set_tps546b24a(REGULATOR_BUS, 0x1A, 1.8)
    tps546b24a.set_tps546b24a(REGULATOR_BUS, 0x1B, 1.8)
    tps546b24a.set_tps546b24a(REGULATOR_BUS, 0x1C, 1.8)



#! /usr/bin/env python3

#
# Copyright (c) 2022 - 2025 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import os
import sys
import subprocess


regulators = {
    'VCCINT': 'tps53681-i2c-0-60',
    'VCC_FPD': 'tps53681-i2c-0-60',
    'VCC_AIE': 'tps53681-i2c-0-61',
    'VCC_IO_SOC': 'tps53681-i2c-0-61',
}

ina226s = {
}

#
# Main routine
#

if len(sys.argv) != 2:
    print("ERROR: missing voltage target")
    quit(-1)

target = sys.argv[1]

if (target in regulators and target in ina226s):
    proc = subprocess.Popen(['/usr/bin/sensors', regulators.get(target), ina226s.get(target)], \
                            stdout=subprocess.PIPE)
elif (target in regulators):
    proc = subprocess.Popen(['/usr/bin/sensors', regulators.get(target)], stdout=subprocess.PIPE)
elif (target in ina226s):
    proc = subprocess.Popen(['/usr/bin/sensors', ina226s.get(target)], stdout=subprocess.PIPE)
else:
    print("ERROR: no information is available for '" + target + "'")
    quit(-1)

total_current = 0

while True:
    line = proc.stdout.readline().decode('ASCII')
    if not line:
        break

    # Voltage
    if (target == 'VCC_FPD' or target == 'VCC_IO_SOC'):
        out = 'vout2:'
    else:
        out = 'vout1:'

    if line.find(out) > -1:
        tokens = line.split()
        if tokens[2] == 'mV':
            voltage = float(tokens[1]) / 1000.0
        else:
            voltage = float(tokens[1])

    # Current
    if (target == 'VCCINT' or target == 'VCC_AIE'):
        out = 'iout1.'
    elif (target == 'VCC_FPD' or target == 'VCC_IO_SOC'):
        out = 'iout2:'
    else:
        out = 'iout1:'

    if line.find(out) > -1:
        tokens = line.split()
        if tokens[2] == 'mA':
            current = float(tokens[1]) / 1000.0
        else:
            current = float(tokens[1])

        if (target == 'VCCINT' or target == 'VCC_AIE'):
            total_current += current

    # Power
    if (target == 'VCCINT' or target == 'VCC_AIE'):
        out = 'pout1:'
    elif (target == 'VCC_FPD' or target == 'VCC_IO_SOC'):
        out = 'pout2:'
    else:
        out = 'power1:'

    if line.find(out) > -1:
        tokens = line.split()
        if tokens[2] == 'mW':
            wattage = float(tokens[1]) / 1000.0
        else:
            wattage = float(tokens[1])


if (target in regulators):
    print('Voltage(V):\t%.4f' % voltage)
    if (target == 'VCCINT' or target == 'VCC_AIE'):
        print('Current(A):\t%.4f' % total_current)
    else:
        print('Current(A):\t%.4f' % current)

if (target in ina226s or target == 'VCCINT' or target == 'VCC_FPD' or \
    target == 'VCC_AIE' or target == 'VCC_IO_SOC'):
    print('Power(W):\t%.4f' % wattage)



#! /usr/bin/env python3

#
# Copyright (c) 2022 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import sys
from periphery import I2C


#
# This routine gets VOUT of a TPS546B24A voltage regulator.
#
def get_tps546b24a(bus, address):
    PMBUS_VOUT_MODE = 0x20
    PMBUS_READ_VOUT = 0x8B

    i2c = I2C(bus)

    # Get the VOUT_MODE
    msgs = [I2C.Message([PMBUS_VOUT_MODE]), I2C.Message([0x0], read = True)]
    i2c.transfer(address, msgs)
    #print("VOUT_MODE: %x" % msgs[1].data[0])
    exponent = ((msgs[1].data[0] & 0x1F) - (4 * 8))

    # Read the VOUT
    msgs = [I2C.Message([PMBUS_READ_VOUT]), I2C.Message([0x00, 0x00], read = True)]
    i2c.transfer(address, msgs)
    mantissa = (msgs[1].data[1] << 8 | msgs[1].data[0])
    #print("data[0] = %x, data[1] = %x" % (msgs[1].data[0], msgs[1].data[1]))
    #print("mantissa = %x" % mantissa)

    voltage = mantissa * (2 ** exponent)

    i2c.close()

    return voltage


#
# Main routine
#

if len(sys.argv) != 2:
    print("ERROR: missing voltage target")
    quit(-1)

target = sys.argv[1]
bus = "/dev/i2c-0"
if target == "VCCO_500":
    address = 0x13
elif target == "VCCO_501":
    address = 0x10
elif target == "VCCO_502":
    address = 0x11
elif target == "VCCO_503":
    address = 0x12
elif target == "LP5_VDDQ_0V5":
    address = 0x14
else:
    print("ERROR: invalid voltage target")
    quit(-1)

voltage = get_tps546b24a(bus, address)
print("Voltage(V):\t%0.2f" % voltage)



#! /usr/bin/env python3

#
# Copyright (c) 2022 - 2023 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import sys
from periphery import I2C


#
# This routine gets VOUT of a TPS53681 voltage regulator.
#
def get_tps53681(bus, address, channel):
    PAGE_ADDR = 0x00
    PMBUS_VOUT_MODE = 0x20
    PMBUS_READ_VOUT = 0x8B

    i2c = I2C(bus)

    # Set the channel to access
    msgs = [I2C.Message([PAGE_ADDR]), I2C.Message([channel], read = False)]
    i2c.transfer(address, msgs)

    # Get the VOUT_MODE to determine the DAC step
    msgs = [I2C.Message([PMBUS_VOUT_MODE]), I2C.Message([0x0], read = True)]
    i2c.transfer(address, msgs)
    if (msgs[1].data[0] & 0x1F) == 0x7:
        DAC_step = 0.005
    elif (msgs[1].data[0] & 0x1F) == 0x3:
        DAC_step = 0.01
    else:
        print("ERROR: invalid VOUT_MODE value")
        quit(-1)

    # Read the VOUT
    msgs = [I2C.Message([PMBUS_READ_VOUT]), I2C.Message([0x00, 0x00], read = True)]
    i2c.transfer(address, msgs)
    #print("data[0] = %x, data[1] = %x" % (msgs[1].data[0], msgs[1].data[1]))

    # The first step is equal to 0.25V with additional steps incremented by DAC_step volts
    if msgs[1].data[0] == 0:
        voltage = 0
    else:
        voltage = (((msgs[1].data[0] - 1) * DAC_step) + 0.25)

    i2c.close()

    return voltage


#
# Main routine
#

if len(sys.argv) != 2:
    print("ERROR: missing voltage target")
    quit(-1)

target = sys.argv[1]
bus = "/dev/i2c-0"
if target == "VCCINT":
    address = 0x60
    channel = 0
elif target == "VCC_CPM5N":
    address = 0x60
    channel = 1
elif target == "VCC_IO_SOC":
    address = 0x61
    channel = 0
elif target == "VCC_FPD":
    address = 0x61
    channel = 1
else:
    print("ERROR: invalid voltage target")
    quit(-1)

voltage = get_tps53681(bus, address, channel)
print("Voltage(V):\t%0.2f" % voltage)



#! /usr/bin/env python3

#
# Copyright (c) 2022 - 2025 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import sys
from periphery import I2C

#
# This routine sets VOUT of a TPS546B24A voltage regulator.
#
def set_tps546b24a(bus, address, voltage):
    PMBUS_OPERATION = 0x01
    PMBUS_VOUT_MODE = 0x20
    PMBUS_VOUT_COMMAND = 0x21

    i2c = I2C(bus)

    # Get the VOUT_MODE
    msgs = [I2C.Message([PMBUS_VOUT_MODE]), I2C.Message([0x0], read = True)]
    i2c.transfer(address, msgs)
    #print("VOUT_MODE: %x" % msgs[1].data[0])
    exponent = ((msgs[1].data[0] & 0x1F) - (4 * 8))
    #print("Exponent: %x" % exponent)

    # Disable VOUT
    msgs = [I2C.Message([PMBUS_OPERATION]), I2C.Message([0x0], read = False)]
    i2c.transfer(address, msgs)

    # Set VOUT
    vout = round(voltage / (2 ** exponent))
    #print("VOUT_COMMAND: %x" % vout)
    msgs = [I2C.Message([PMBUS_VOUT_COMMAND]), I2C.Message([(vout & 0xFF), (vout >> 8)], read = False)]
    i2c.transfer(address, msgs)

    # Enable VOUT
    msgs = [I2C.Message([PMBUS_OPERATION]), I2C.Message([0x80], read = False)]
    i2c.transfer(address, msgs)

    i2c.close()

    return


#
# Main routine
#
if __name__ == '__main__':
    if len(sys.argv) != 3:
        print("ERROR: missing regulator address and voltage value")
        quit(-1)

    bus = "/dev/i2c-0"
    address = sys.argv[1]
    value = sys.argv[2]

    set_tps546b24a(bus, int(address, 16), float(value))



#! /usr/bin/env python3

#
# Copyright (c) 2022 - 2025 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import sys
from periphery import I2C

#
# This routine sets VOUT of a TPS53681 voltage regulator.
#
def set_tps53681(bus, address, channel, voltage):
    PAGE_ADDR = 0x00
    PMBUS_OPERATION = 0x01
    PMBUS_VOUT_MODE = 0x20
    PMBUS_VOUT_COMMAND = 0x21

    i2c = I2C(bus)

    # Set the channel to access
    msgs = [I2C.Message([PAGE_ADDR]), I2C.Message([channel], read = False)]
    i2c.transfer(address, msgs)

    # Get the VOUT_MODE to determine the DAC step
    msgs = [I2C.Message([PMBUS_VOUT_MODE]), I2C.Message([0x0], read = True)]
    i2c.transfer(address, msgs)
    if (msgs[1].data[0] & 0x1F) == 0x7:
        DAC_step = 0.005
    elif (msgs[1].data[0] & 0x1F) == 0x1:
        # VOUT_MODE is not set correctly on TPS53681 parts on this board.  It is
        # reading 0x1 instead of 0x7.
        DAC_step = 0.005
    elif (msgs[1].data[0] & 0x1F) == 0x4:
        DAC_step = 0.01
    else:
        print("ERROR: invalid VOUT_MODE value")
        quit(-1)

    # Disable VOUT
    msgs = [I2C.Message([PMBUS_OPERATION]), I2C.Message([0x0], read = False)]
    i2c.transfer(address, msgs)

    # The first step is equal to 0.25V with additional steps incremented by DAC_step volts
    VID = ((voltage - 0.25) / DAC_step) + 1
    msgs = [I2C.Message([PMBUS_VOUT_COMMAND]), I2C.Message([int(VID), 0x0], read = False)]
    i2c.transfer(address, msgs)

    # Enable VOUT
    msgs = [I2C.Message([PMBUS_OPERATION]), I2C.Message([0x80], read = False)]
    i2c.transfer(address, msgs)

    i2c.close()

    return


#
# Main routine
#
if len(sys.argv) != 3:
    print("ERROR: missing voltage target and value")
    quit(-1)

target = sys.argv[1]
value = sys.argv[2]
bus = "/dev/i2c-0"
if target == "VCCINT":
    address = 0x60
    channel = 0
elif target == "VCC_FPD":
    address = 0x60
    channel = 1
elif target == "VCC_AIE":
    address = 0x61
    channel = 0
elif target == "VCC_SOC":
    address = 0x61
    channel = 1
else:
    print("ERROR: invalid voltage target")
    quit(-1)

set_tps53681(bus, address, channel, float(value))



#! /usr/bin/env python3

#
# Copyright (c) 2025 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import os
import sys
import time
import glob
import subprocess
from smbus2 import SMBus, i2c_msg

Runtime_Ext = ".csv"
EEPROM_Ext = ".eeprom.py"

Clock_Dir = "/usr/share/system-controller-app/.sc_app/vendor_clock/"
Default_CF_Dir = "/usr/share/system-controller-app/BIT/clock_files/"
Custom_CF_Dir = "/data/clock_files/"

global EEPROM_Size, EEPROM_Device_ID, EEPROM_Config_Word, EEPROM_Page_ID

#
# This function reads in a file with 'Runtime_Ext' format into a list
# of integer data.  The file format is two hex data per line represented
# in ASCII.
#
def Convert_EEPROM(File):
    Data = []
    try:
        with open(File, 'r') as f:
            for Line in f:
                Hex_Str = Line.strip()  # Remove leading/trailing whitespace and newlines
                Hex_Str = Hex_Str[2:]
                try:
                    Byte_Data = bytes.fromhex(Hex_Str)
                    Byte_List = list(Byte_Data)
                    for Byte in Byte_List:
                        Data.append(Byte)
                except:
                    print("ERROR: invalid hex value on line " + line.strip() + " of '" + File + "'")
                    exit(-1)
    except:
        print("ERROR: failed to open clock file '" + File + "'")
        exit(-1)
    return Data


#
# This function returns all the files with 'EEPROM_Ext' extension
# that are found in 'Directory' argument.
#
def EEPROM_Files(Directory):
    Files = []
    if os.path.exists(Directory):
        for File in os.listdir(Directory):
            if File.endswith(EEPROM_Ext):
                File_Path = os.path.join(Directory, File)
                Files.append(File_Path)
    return Files


#
# This function return True if the content of argument 'File' matches
# the content of EEPROM.
#
def EEPROM_Compare(File, I2C_Bus, I2C_Addr, Mode):
    if Mode == 0:
        EEPROM_Data = Convert_EEPROM(File)
    elif Mode == 1:
        # Assume EEPROM is blank if its first 256 bytes are 0xFF
        EEPROM_Data = [0xFF] * 256
    else:
        print("ERROR: invalid mode of EEPROM comparison")
        exit(-1)

    try:
        with SMBus(I2C_Bus, force=True) as Bus:
            Size = 32
            for I in range(0, (len(EEPROM_Data) - Size), Size):
                Offset = [((I >> 8) & 0xFF), (I & 0xFF)]
                Write_Msg = i2c_msg.write(I2C_Addr, Offset)
                Read_Msg = i2c_msg.read(I2C_Addr, Size)
                Bus.i2c_rdwr(Write_Msg, Read_Msg)
                Data = list(Read_Msg)
                for J in range(0, Size):
                    if Data[J] != EEPROM_Data[I + J]:
                        return False
            return True
    except:
        print("ERROR: failed to compare EEPROM with design '" + File + "'")
        exit(-1)


#
# This function programs EEPROM with a new clock design.
#
def Program_EEPROM(I2C_Bus, I2C_Addr, Clock_Design):
    EEPROM_Data = Convert_EEPROM(Clock_Design)
    try:
        with SMBus(I2C_Bus, force=True) as Bus:
            Size = 32
            for I in range(0, len(EEPROM_Data), Size):
                Addr_Data = [((I >> 8) & 0xFF), (I & 0xFF)] + EEPROM_Data[I:(I + Size)]
                Msg = i2c_msg.write(I2C_Addr, Addr_Data)
                Bus.i2c_rdwr(Msg)
                time.sleep(0.01)
    except:
        print("ERROR: failed to program EEPROM with design '" + Clock_Design + "'")
        exit(-1)


#
# This function programs a new runtime design on a clock.
#
def Program_Clock(I2C_Bus, I2C_Addr, Clock_Name, New_Design):
    try:
        with open(New_Design, 'r') as f:
            Bus = SMBus(I2C_Bus, force=True)
            for Line in f:
                if Line.startswith('sleep'):
                    #time.sleep(0.01)
                    continue
                Offset, Data = Line.split(',')
                Addr_Data = [int(Offset, 16), int(Data, 16)]
                Msg = i2c_msg.write(I2C_Addr, Addr_Data)
                Bus.i2c_rdwr(Msg)
    except:
        print("ERROR: failed to open file '" + New_Design + "'")
        exit(-1)

    #
    # Record the new programmed design in 'Clock_Dir + Clock_Name' file
    # so function 'Discover_Clock' could reference it.
    #
    os.makedirs(Clock_Dir, exist_ok=True)
    try:
        with open(Clock_Dir + Clock_Name, 'w') as f:
            f.write(os.path.basename(New_Design.strip(Runtime_Ext)))
    except:
        print("ERROR: failed to create file '" + (Clock_Dir + Clock_Name) + "'")
        exit(-1)


#
# This function identifies which clock design is in-effect.
#
def Discover_Clock(I2C_Bus, I2C_Addr, Chip, Clock_Name, Default_Design):
    #
    # If the clock has already been custom-programmed, there will be a file
    # named after that clock under 'Clock_Dir' directory containing the design
    # name.
    #
    if os.path.exists(Clock_Dir + Clock_Name):
        try:
            with open(Clock_Dir + Clock_Name) as f:
                print(f.readline())
                return
        except:
            print("ERROR: failed to open '" + (Clock_Dir + Clock_Name) + "'")
            exit(-1)

    #
    # In case EEPROM has not previously been programmed, its content is all 0xFF.
    #
    if EEPROM_Compare("", I2C_Bus, I2C_Addr, 1):
        print(Default_Design)
        return

    #
    # In case there has not been previous custom design programming, find all
    # possible clock designs that are available for the clock.  Compare each
    # with current content of EEPROM to find out which design has been in-effect
    # since boot time.
    #
    Files = EEPROM_Files(Default_CF_Dir + Chip)
    Files += EEPROM_Files(Custom_CF_Dir)
    for File in Files:
        if EEPROM_Compare(File, I2C_Bus, I2C_Addr, 0):
            print(os.path.basename(File.strip(EEPROM_Ext)))
            return
    print("ERROR: could not identify the design for clock '" + Clock_Name + "'")
    exit(-1)


#
# Find a clock file
#
def Find_Clock_File(Chip, Design, Extension):
    File = Default_CF_Dir + Chip + "/" + Design + Extension
    if not (os.path.exists(File)):
        File = Custom_CF_Dir + Design + Extension
        if not (os.path.exists(File)):
            return None

    #
    # Validate that the design file for programming EEPROM is generated correctly
    #
    if (Extension == EEPROM_Ext):
        Command = ['/bin/head', '-1', File]
        Result = subprocess.run(Command, capture_output=True, text=True)
        EEPROM_Identifier = Result.stdout.strip().splitlines()[0]

        #
        # Strip the leading '0x' and convert the remaining into individual 2-character strings
        #
        First_Line = EEPROM_Identifier[2:]
        First_Four_Bytes = [First_Line[i:i+2] for i in range(0, len(First_Line), 2)]
        EEPROM_Identifier = [EEPROM_Size, EEPROM_Device_ID, EEPROM_Config_Word, EEPROM_Page_ID]
        if (First_Four_Bytes != EEPROM_Identifier):
            print("ERROR: file '" + File + "' is not generated correctly")
            exit(-1)

    return File


#
# Identify board revision
#
def Get_Board_Revision(FRU_Path):
    Arg = "--fru-file=" + FRU_Path
    Command = ['/usr/sbin/ipmi-fru', Arg]
    Result = subprocess.run(Command, capture_output=True, text=True)
    OnBoard_EEPROM = Result.stdout.strip().splitlines()
    Label = "  FRU Board Custom Info: "
    Revisions = ['A01']
    for Rev in Revisions:
        if Label + Rev in OnBoard_EEPROM:
            return Rev

    return "UNKNOWN"


#
# Main Function
#
Num_Args = len(sys.argv) - 1
if Num_Args < 6:
    print("ERROR: missing the required number of arguments")
    exit(-1)

Chip = os.path.basename(sys.argv[0].strip(".py"))
Board = sys.argv[1]
I2C_Bus = int(sys.argv[2].replace("/dev/i2c-", ""))
I2C_Addr = int(sys.argv[3])
Default_Design = sys.argv[4]
Command = sys.argv[5]
Clock_Name = sys.argv[6]

SiTime_EEPROM_Search = "/sys/bus/i2c/devices/*/eeprom_sit95*/nvmem"
Bus_Address = glob.glob(SiTime_EEPROM_Search, recursive=True)[0].split('/')[5]
EEPROM_I2C_Addr = int(Bus_Address[Bus_Address.find('-') + 1:], 16)

#
# Different revisions of the board have different default clock design
#
OnBoard_EEPROM_Search = "/sys/bus/i2c/devices/*/eeprom_cc*/nvmem"
Board_Revision = "-" + Get_Board_Revision(glob.glob(OnBoard_EEPROM_Search, recursive=True)[0])
Default_Design_Search = Default_CF_Dir + Chip + "/" + Board + Board_Revision + "*"
if (len(glob.glob(Default_Design_Search)) != 0):
    Index = Default_Design.find(Board)
    Default_Design = Default_Design[:Index + len(Board)] + Board_Revision + \
                     Default_Design[Index + len(Board):]

#
# EEPROM identifier data
#
EEPROM_Size= "00"
EEPROM_Device_ID = "69"
EEPROM_Config_Word = "01"
EEPROM_Page_ID = "00"


if (Command == "setclock" or Command == "setbootclock"):
    if not Num_Args == 7:
        print("ERROR: invalid number of arguments")
        exit(-1)
    else:
        Clock_Design = sys.argv[7]

if Command == "getclock":
    Discover_Clock(I2C_Bus, EEPROM_I2C_Addr, Chip, Clock_Name, Default_Design)

elif Command == "setclock":
    Runtime_File = Find_Clock_File(Chip, Clock_Design, Runtime_Ext)
    if not Runtime_File:
        print("ERROR: failed to find design clock file '" + Clock_Design + "'")
        exit(-1)

    Program_Clock(I2C_Bus, I2C_Addr, Clock_Name, Runtime_File)

elif Command == "setbootclock":
    Runtime_File = Find_Clock_File(Chip, Clock_Design, Runtime_Ext)
    if not Runtime_File:
        print("ERROR: failed to find design clock file '" + Clock_Design + "'")
        exit(-1)

    EEPROM_File = Find_Clock_File(Chip, Clock_Design, EEPROM_Ext)
    if not EEPROM_File:
        print("ERROR: failed to find EEPROM clock file '" + Clock_Design + "'")
        exit(-1)

    Program_EEPROM(I2C_Bus, EEPROM_I2C_Addr, EEPROM_File)
    Program_Clock(I2C_Bus, I2C_Addr, Clock_Name, Runtime_File)

elif Command == "restoreclock":
    Runtime_File = Find_Clock_File(Chip, Default_Design, Runtime_Ext)
    if not Runtime_File:
        print("ERROR: failed to find design clock file '" + Default_Design + "'")
        exit(-1)

    EEPROM_File = Find_Clock_File(Chip, Default_Design, EEPROM_Ext)
    if not EEPROM_File:
        print("ERROR: failed to find EEPROM clock file '" + Default_Design + "'")
        exit(-1)

    Program_EEPROM(I2C_Bus, EEPROM_I2C_Addr, EEPROM_File)
    Program_Clock(I2C_Bus, I2C_Addr, Clock_Name, Runtime_File)

else:
    print("ERROR: invalid clock command")
    exit(-1)



#! /usr/bin/env python3

#
# Copyright (c) 2022 Xilinx, Inc.  All rights reserved.
# Copyright (c) 2022 - 2025 Advanced Micro Devices, Inc.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import sys
import time
import subprocess
from chipscopy import create_session

if len(sys.argv) != 5:
    print("ERROR: missing DDRMC number, board name, image id, and unique id")
    quit(-1)

DDRMC = int(sys.argv[1])
BOARD_NAME = str(sys.argv[2])
image_id = int(sys.argv[3], 16)
image_uid = int(sys.argv[4], 16)

# Specify locations of the running hw_server and cs_server below
CS_URL = os.getenv("CS_SERVER_URL", "TCP:localhost:3042")
HW_URL = os.getenv("HW_SERVER_URL", "TCP:localhost:3121")

# Read the Versal UUID registers
def get_unique_id (versal_device, image_id):
    # Read the ImageInfoTable Address
    ImageInfoTableAddr = 0xF2014040
    offset = versal_device.memory_read(ImageInfoTableAddr)[0]
    if (offset == 0xdeadbeef or offset == 0):
        return 0xdeadbeef

    # Get the length of the ImageInfo Table
    length = versal_device.memory_read(ImageInfoTableAddr + 8)[0] & 0xffff

    # Parse the image info table until we find the requested image_id and return the UUID
    img = 0
    while img < length:
        image_id_reg = versal_device.memory_read(offset)[0]
        if image_id_reg == image_id:
            return versal_device.memory_read(offset + 4)[0]
        img += 1
        offset = offset + 16

    return 0xdeadbeef

with create_session(cs_server_url=CS_URL, hw_server_url=HW_URL, pre_device_scan_delay=3000) as session:
    # XXX- a workaround for early release of labtools binaries for T50
    if BOARD_NAME == "VEK385":
        session = create_session(cs_server_url=CS_URL, hw_server_url=HW_URL)

    check_for_device = True
    start_time = time.time()
    while check_for_device:
        try:
            versal_device = session.devices.filter_by(family="versal").get()
            check_for_device = False
        except ValueError:
            if time.time() - start_time > 5:
                print("ERROR: no device found on the JTAG chain")
                quit(-1)

    PDI_FILE = "/data/PDIs/default.pdi"

    unique_id = get_unique_id(versal_device, image_id)
    if (image_uid != unique_id):
        # Program the PDI
        versal_device.program(PDI_FILE)
    else:
        print("PDI already loaded")

    # Discover debug cores
    versal_device.discover_and_setup_cores()

    # Get the target DDRMC
    DDR = versal_device.ddrs[(DDRMC - 1)]
    if not DDR.is_enabled:
        print("ERROR: DDRMC " + str(DDRMC) + " is not enabled")
        quit(-1)

    print("Calibration Status:", DDR.get_cal_status())



#! /usr/bin/env python3

#
# Copyright (c) 2023 - 2024 Advanced Micro Devices, Inc.  All rights reserved.
#
# SPDX-License-Identifier: MIT
#

import sys
import argparse
import time
from smbus2 import SMBus, i2c_msg

def i2c_write(bus, dev, reg, val):
    data = [reg & 0xff]
    data += val
    msg = i2c_msg.write(dev, data)
    try:
        bus.i2c_rdwr(msg)
    except:
        print("Error accessing I2C device %02x" % dev)
        exit(-1)

def i2c_read(bus, dev, reg, size):
    data = [reg & 0xff]
    write = i2c_msg.write(dev, data)
    read = i2c_msg.read(dev, size)
    try:
        bus.i2c_rdwr(write, read)
    except:
        print("Error accessing I2C device %02x" % dev)
        exit(-1)
    return list(read)

def i2c_write_a16(bus, dev, reg, val):
    data = [reg >> 8, reg & 0xff]
    data += val
    msg = i2c_msg.write(dev, data)
    try:
        bus.i2c_rdwr(msg)
    except:
        print("Error accessing I2C device %02x" % dev)
        exit(-1)

def i2c_read_a16(bus, dev, reg, size):
    data = [reg >> 8, reg & 0xff]
    write = i2c_msg.write(dev, data)
    read = i2c_msg.read(dev, size)
    try:
        bus.i2c_rdwr(write, read)
    except:
        print("Error accessing I2C device %02x" % dev)
        exit(-1)
    return list(read)

EEPROM_PAGE=0xCF
EEPROM_I2C_ADDR=0x68+0
EEPROM_SIZE=0x68+1
EEPROM_OFFSET=0x68+2
EEPROM_CMD=0x68+4
EEPROM_BUF=0x80

EEPROM_STAT_PAGE=0xc0
EEPROM_STAT_REG=0x14+0x08

def wait_ready(bus, dev):
    i2c_write(bus, dev, 0xFC, [0x00, EEPROM_STAT_PAGE, 0x10, 0x20])
    cmd = [0, 0]
    MAX_RETRY=500
    retry=0
    while cmd == [0, 0] and retry < MAX_RETRY:
        cmd = i2c_read(bus, dev, EEPROM_STAT_REG, 2)
        retry += 1

    if retry == MAX_RETRY:
        return -1

    i2c_write(bus, dev, 0xFC, [0x00, EEPROM_PAGE, 0x10, 0x20])
    return 0

def erase_clock(busnum, dev):
    bus=SMBus(busnum)

    # Reset the page address.  supposed to do this once after reset
    i2c_write(bus, dev, 0xFC, [0x00, EEPROM_PAGE, 0x10, 0x20])

    eep_addr = 0

    data=[]
    for i in range(0,128):
        data.append(0xff)

    i2c_write(bus, dev, EEPROM_BUF, data)

    #Set eeprom address to 0x54
    i2c_write(bus, dev, EEPROM_I2C_ADDR, [0x50 + ((eep_addr & 0x10000)>>14)])
    #set write size
    i2c_write(bus, dev, EEPROM_SIZE, [128])
    #set eeprom offset
    i2c_write(bus, dev, EEPROM_OFFSET, [eep_addr & 0xff, (eep_addr >> 8) & 0xff])
    #set the write command
    i2c_write(bus, dev, EEPROM_CMD, [0x02, 0xEE])

    if wait_ready(bus, dev) != 0:
        print("1 Error accessing EEPROM")
        exit(-1)

    eep_addr = 0xF000

    #Set eeprom address to 0x54
    i2c_write(bus, dev, EEPROM_I2C_ADDR, [0x50 + ((eep_addr & 0x10000)>>14)])
    #set write size
    i2c_write(bus, dev, EEPROM_SIZE, [128])
    #set eeprom offset
    i2c_write(bus, dev, EEPROM_OFFSET, [eep_addr & 0xff, (eep_addr >> 8) & 0xff])
    #set the write command
    i2c_write(bus, dev, EEPROM_CMD, [0x02, 0xEE])

    if wait_ready(bus, dev) != 0:
        print("2 Error accessing EEPROM")
        exit(-1)

    return 0

def verify_clock(filename, busnum, dev):
    try:
        with open(filename, 'rb') as file:
            eep_dat = file.read()
    except:
        print("Failed to open file", filename)
        exit(-1)

    bus=SMBus(busnum)

    # Reset the page address.  supposed to do this once after reset
    i2c_write(bus, dev, 0xFC, [0x00, EEPROM_PAGE, 0x10, 0x20])

    eep_addr = 0
    while eep_addr < len(eep_dat):
        remain = len(eep_dat) - eep_addr
        if (remain < 128):
            size = remain
        else:
            size = 128

        data=[]
        for i in range(0,size):
            data.append(eep_dat[i + eep_addr])

        #Set eeprom address to 0x54
        i2c_write(bus, dev, EEPROM_I2C_ADDR, [0x50 + ((eep_addr & 0x10000)>>14)])
        #set write size
        i2c_write(bus, dev, EEPROM_SIZE, [size])
        #set eeprom offset
        i2c_write(bus, dev, EEPROM_OFFSET, [eep_addr & 0xff, (eep_addr >> 8) & 0xff])
        #set the write command
        i2c_write(bus, dev, EEPROM_CMD, [0x01, 0xEE])

        if wait_ready(bus, dev) != 0:
            print("Error accessing EEPROM")
            exit(-1)

        print("Addr %04x:%04x" % (eep_addr, len(eep_dat))) 

        check_data = i2c_read(bus, dev, EEPROM_BUF, size)
        for i in range(0, size):
            if data[i] != check_data[i]:
                print("Data mismatch at %05x, got %02x expected %02x" %(eep_addr+i, check_data[i], data[i]))
                exit(1)

        eep_addr += size
    print("Verification Complete")
    return 0

def program_clock(filename, busnum, dev):
    try:
        with open(filename, 'rb') as file:
            eep_dat = file.read()
    except:
        print("Failed to open file", filename)
        exit(-1)

    bus=SMBus(busnum)

    # Reset the page address.
    i2c_write(bus, dev, 0xFC, [0x00, EEPROM_PAGE, 0x10, 0x20])

    eep_addr = 0
    while eep_addr < len(eep_dat):
        remain = len(eep_dat) - eep_addr
        if (remain < 128):
            size = remain
        else:
            size = 128

        data=[]
        for i in range(0,size):
            data.append(eep_dat[i + eep_addr])

        i2c_write(bus, dev, EEPROM_BUF, data)

        #Set eeprom address to 0x54
        i2c_write(bus, dev, EEPROM_I2C_ADDR, [0x50 + ((eep_addr & 0x10000)>>14)])
        #set write size
        i2c_write(bus, dev, EEPROM_SIZE, [size])
        #set eeprom offset
        i2c_write(bus, dev, EEPROM_OFFSET, [eep_addr & 0xff, (eep_addr >> 8) & 0xff])
        #set the write command
        i2c_write(bus, dev, EEPROM_CMD, [0x02, 0xEE])

        if wait_ready(bus, dev) != 0:
            print("Error accessing EEPROM")
            exit(-1)
        print("Addr %04x:%04x" % (eep_addr, len(eep_dat)))
        eep_addr += size

    # Reset chip
    i2c_write(bus, dev, 0xFC, [0x00, 0xC0, 0x10, 0x20])
    i2c_write(bus, dev, 0x12, [0x5a])
    print("Programming Complete")
    return 0


def auto_int(x):
    return int(x, 0)

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', dest='filename', help="Clock programming bin file")
    parser.add_argument('-b', dest='bus', type=auto_int, default=11, help="I2C Bus number")
    parser.add_argument('-d', dest='dev', type=auto_int, default=0x5b, help="I2C Device Address")
    parser.add_argument('-v', dest='verify', default=False, action=argparse.BooleanOptionalAction, help="verify")
    parser.add_argument('-e', dest='erase', default=False, action=argparse.BooleanOptionalAction, help="erase")
    args = parser.parse_args()

    if (args.verify):
        print(f"Filename %s, bus %d, dev %02x" % (args.filename, args.bus, args.dev))
        ret = verify_clock(args.filename, args.bus, args.dev)
    elif (args.erase):
        print(f"Erase bus %d, dev %02x" % (args.bus, args.dev))
        ret = erase_clock(args.bus, args.dev)
    else:
        print(f"Filename %s, bus %d, dev %02x" % (args.filename, args.bus, args.dev))
        ret = program_clock(args.filename, args.bus, args.dev)

    exit(ret)
